from typing import TYPE_CHECKING
if not TYPE_CHECKING:
    import arclet.cesloi.event.inserter
    import arclet.cesloi.event.mirai
    import arclet.cesloi.event.messages
    import arclet.cesloi.event.lifecycle
    import arclet.cesloi.event.base
